using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using System;

//////--------------------------------------------------------------------------------//
//////            Автор плагина Vladimir.Kzi / Plugin autor Vladimir.Kzi              //
//////--------------------------------------------------------------------------------//
//////  Поддержка данного плагина: www.oxide-russia.ru and www.vk.com/deviantplugins  //
//////--------------------------------------------------------------------------------//
//////  Support for this plugin: www.oxide-russia.ru и www.vk.com/deviantplugins      //
//////--------------------------------------------------------------------------------//

namespace Oxide.Plugins
{
    [Info("SpeedQuest", "Vladimir.Kzi", "0.0.7")]
	[Description("Speed quest in chat with reward.")]
	class SpeedQuest : HurtworldPlugin
	{
		
		System.Random random = new System.Random();
		string questAnswer;
		
		//////////////////////////////////////////////////////////////////////////////////////////
        // Классы
        //////////////////////////////////////////////////////////////////////////////////////////
		
		private ConfigData configData;
		
		class ConfigData
        {
            public bool autoStart { get; set; }
			public float intervalAutoStart { get; set; }
			public int symbolCount { get; set; }
            public int minPlayers { get; set; }
			public bool giveGift { get; set; }
			public bool writeWinLog { get; set; }
			public bool showWinConsole { get; set; }
            public List<GiftItems> listGiftItems{ get; set; }
            public List<object> listAlphabet{ get; set; }
        }
		
		class GiftItems
        {
            public string itemName;
            public string ItemIDNameGuID;
            public int itemCount;
        }
		
		class ItemsList
        {
            public ItemGeneratorAsset ItemGen;
			public string ItemName;
            public int ItemId;
			public string ItemGuID;
        }
		
		List<ItemsList> allItemsList = new List<ItemsList>();
		
		//////////////////////////////////////////////////////////////////////////////////////////
        // Загрузка плагина
        //////////////////////////////////////////////////////////////////////////////////////////
		
		void Loaded()
		{
			LoadVariables();
			LoadMessages();
			
			RegisterPerm("admin");
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        ///     Конфиг
        ////////////////////////////////////////////////////////////////////////////////////////
		
		protected override void LoadDefaultConfig()
        {
            PrintWarning("Generating new configuration file for " + this.Title);
            var config = new ConfigData
            {
				autoStart = true,
				intervalAutoStart = 30f,
				symbolCount = 12,
				minPlayers = 3,
				giveGift = true,
				writeWinLog = false,
				showWinConsole = false,
				listGiftItems = new List<GiftItems>
				{
					{
						new GiftItems
						{
							itemName = "C4",
							ItemIDNameGuID = "288",
							itemCount = 1,
						}
					},
					{
						new GiftItems
						{
							itemName = "Detonator",
							ItemIDNameGuID = "d31480f29cb72f944b0c1eea1caf81b5",
							itemCount = 1,
						}
					},
					{
						new GiftItems
						{
							itemName = "AR RedDot Vista",
							ItemIDNameGuID = "AR RedDot Vista",
							itemCount = 1,
						}
					},
				},
				listAlphabet = new List<object> 
				{
					"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","R","S","T","U","V","X","W","Y","Z"
				}
			};
            SaveConfig(config);
		}
		
		////////////////////////////////////////
		///     Языковые Сообщения
		////////////////////////////////////////  //
		
		void LoadMessages()
		{
			// English
			lang.RegisterMessages(new Dictionary<string, string>
			{
				{"msg_Prefix", "<color=lime>[SpeedQuest]</color>"},
				{"msg_NoPermission", "You do not have permission for this command."},
				{"msg_Quest", "<color=orange>The first player that writes:</color> <color=green>{ans}</color> <color=orange>will win the prize.</color>"},
				{"msg_Winner", "<color=green>{Name}</color> was the first one to write correctly and receives a prize <color=red>{gift}</color>(<color=orange>x{cout}</color>)!"}
			}, this);

			// Polish
			lang.RegisterMessages(new Dictionary<string, string>
			{
				{"msg_Prefix", "<color=orange>[SpeedQuest]</color>"},
				{"msg_NoPermission", "Nie posiadasz permisji do użycia tej komendy"},
				{"msg_Quest", "<color=orange>Pierwszy gracz który przepisze</color> <color=green>{ans}</color> <color=orange>otrzyma nagrodę.</color>"},
				{"msg_Winner", "<color=green>{Name}</color> był pierwszy który dobrze przepisał i został nagrodzony :<color=red>{gift}</color>(<color=orange>x{cout}</color>)!"}
			}, this, "pl");
		}
		
		void OnServerInitialized()
		{
			questAnswer = null;
			
			AllItemsList();
			
			if (configData.autoStart)
			{
				timer.Repeat(configData.intervalAutoStart * 60, 0, cmdQuest);
			}
		}
		
		void cmdQuest()
		{
			if(!getMinPlayers())
			{
				return;
			}
			
			string rString = "";
			for (int i = 0; i < configData.symbolCount; i++ )
			{
				rString = rString + configData.listAlphabet[random.Next(0, configData.listAlphabet.Count-1)].ToString();
			}
			
			questAnswer = rString;
			BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Quest").Replace("{ans}", questAnswer.ToString())); 
			return;	  
		}
		
		void OnPlayerChat(PlayerSession session, string message)
		{
			if(questAnswer == null)
			{
				return;
			}
			
			if( message == questAnswer)
			{						
				if(configData.giveGift)
				{
					int	rand = UnityEngine.Random.Range(0, configData.listGiftItems.Count);
					GiftItems ItemGift = configData.listGiftItems[rand];
					string pName = "";
					pName = session.Identity.Name;
					
					if((bool)this.GiveItem(session, ItemGift.ItemIDNameGuID, ItemGift.itemCount))
					{
						BroadcastChat(GetMsg("msg_Prefix"), GetMsg("msg_Winner").Replace("{Name}", pName).Replace("{gift}", ItemGift.itemName.ToString()).Replace("{cout}", ItemGift.itemCount.ToString()));
						if (configData.writeWinLog)
						{
							LogToFile("SpeedQuest", $"["+System.DateTime.Now+"] "+pName+" ("+session.SteamId+") GIFT: "+ItemGift.itemName.ToString()+" IDorNAMEorGUID: "+ItemGift.ItemIDNameGuID.ToString()+" COUNT:"+ItemGift.itemCount.ToString(), this, true);
						}
						if (configData.showWinConsole)
						{
							Puts(pName+" ("+session.SteamId+") GIFT: "+ItemGift.itemName.ToString()+" IDorNAMEorGUID: "+ItemGift.ItemIDNameGuID.ToString()+" COUNT:"+ItemGift.itemCount.ToString());
						}
					}
				}
				questAnswer = null;
			}
			
			return;
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Создаем список всех предметов( Один раз за запуск сервера )
        ////////////////////////////////////////////////////////////////////////////////////////
		
		void AllItemsList()
		{
			foreach (var item in GlobalItemManager.Instance.GetGenerators())
			{
				if (item.Value?.name != null)
				{
					ItemsList aitem = new ItemsList();
					aitem.ItemGen = item.Value;
					aitem.ItemName = item.Value.name;
					aitem.ItemId = item.Value.GeneratorId;
					aitem.ItemGuID = RuntimeHurtDB.Instance.GetGuid(item.Value);
					
					allItemsList.Add(aitem);
				}
			}
		}
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Ищем нужный предмет в списке всех предметов
        ////////////////////////////////////////////////////////////////////////////////////////
		
		ItemGeneratorAsset FindItem(string itemNameOrIdOrGuid)
        {
			List<ItemsList> items = allItemsList;
            ItemsList aitem;
			ItemGeneratorAsset item;
            int itemId;
			
            if (int.TryParse(itemNameOrIdOrGuid, out itemId))
            {
                aitem = items.First(i => i.ItemId == itemId);
				item = aitem.ItemGen;
            }
			else if((from x in items where x.ItemGuID == itemNameOrIdOrGuid select x).Count() > 0)
            {
                aitem = items.First(i => i.ItemGuID.Contains(itemNameOrIdOrGuid));
				item = aitem.ItemGen;
            }
            else if((from x in items where x.ItemName.ToLower() == itemNameOrIdOrGuid.ToLower() select x).Count() > 0)
            {
                aitem = items.First(i => i.ItemName.ToLower().Contains(itemNameOrIdOrGuid.ToLower()));
				item = aitem.ItemGen;
            }
			else
            {
				item = null;
            }
			
            return item;
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Выдаем игроку нужный предмет
        ////////////////////////////////////////////////////////////////////////////////////////

		object GiveItem(PlayerSession session, string itemNameOrIdOrGuid, int amount = 1)
        {
            string itemName = itemNameOrIdOrGuid;
            PlayerInventory inventory = session.WorldPlayerEntity.GetComponent<PlayerInventory>();
            ItemGeneratorAsset generator = FindItem(itemNameOrIdOrGuid);
			
			if(generator == null)
			{
				PrintWarning("Something is wrong with the item:" + itemNameOrIdOrGuid);
				return false;
			}

            ItemObject itemObj;
            if (generator.IsStackable())
            {
                itemObj = GlobalItemManager.Instance.CreateItem(generator, amount);
                if (!inventory.GiveItemServer(itemObj))
                {
                    GlobalItemManager.SpawnWorldItem(itemObj, inventory);
                }
            }
            else
            {
                int amountGiven = 0;
                while (amountGiven < amount)
                {
                    itemObj = GlobalItemManager.Instance.CreateItem(generator);
                    if (!inventory.GiveItemServer(itemObj))
                    {
                        GlobalItemManager.SpawnWorldItem(itemObj, inventory);
                    }
                    amountGiven++;
                }
            }
			
            return true;
        }
		
		////////////////////////////////////////////////////////////////////////////////////////
        //    Проверяем достаточное ли количество игроков онлайн 
        ////////////////////////////////////////////////////////////////////////////////////////
		
		bool getMinPlayers()
        {
            var allPlayers = GameManager.Instance.GetSessions().Values.ToList();
            if (allPlayers.Count() > configData.minPlayers)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
		
		////////////////////////////////////////
        ///     Converting
        ////////////////////////////////////////

        string ListToString(List<string> list, int first, string seperator) => string.Join(seperator, list.Skip(first).ToArray());
		
		//////////////////////////////////////////////////////////////////////////////////////////
        /////      Функции Данных конфига, "запись(замена)" и "чтение"...
        //////////////////////////////////////////////////////////////////////////////////////////
		
        private void LoadVariables()
        {
            LoadConfigVariables();
            SaveConfig();
        }
		
        private void LoadConfigVariables()
        {
            configData = Config.ReadObject<ConfigData>();
        }
		
		private void SaveConfig(ConfigData config)
        {
            Config.WriteObject(config, true);
        }

        /////////////////////////////////////////////////////////////////////////////////////////
		
		////////////////////////////////////////
		///     Разрешения (permissions, пермишены)
		////////////////////////////////////////
		
		void RegisterPerm(params string[] permArray)
		{
			string perm = ListToString(permArray.ToList(), 0, ".");
			
			permission.RegisterPermission($"{PermissionPrefix}.{perm}", this);
		}
		
		bool HasPerm(object uid, params string[] permArray)
		{
			uid = uid.ToString();
			string perm = ListToString(permArray.ToList(), 0, ".");
			
			return permission.UserHasPermission(uid.ToString(), $"{PermissionPrefix}.{perm}");
		}
		
		string PermissionPrefix
		{
			get
			{
				return this.Title.Replace(" ", "").ToLower();
			}
		}
		
		//////////////////////////////////////////////////////////////////////////////////////
        ///     ЧАТ
        //////////////////////////////////////////////////////////////////////////////////////
		
		string GetMsg(string key, string userID = "") => lang.GetMessage(key, this, userID);
		
		void BroadcastChat(string prefix, string msg) => Server.Broadcast(msg, prefix);

        void SendChatMessage(PlayerSession player, string prefix, string msg) => hurt.SendChatMessage(player, prefix, msg);
		
	}
}